@@ -20,7 +20,7 @@ import pretend
 import pytest
 
 from twine.commands import upload
-from twine import package, cli
+from twine import package, cli, exceptions, settings
 import twine
 
 import helpers
@@ -28,48 +28,42 @@ import helpers
 WHEEL_FIXTURE = 'tests/fixtures/twine-1.5.0-py2.py3-none-any.whl'
 
 
-def test_ensure_wheel_files_uploaded_first():
-    files = upload.group_wheel_files_first(["twine/foo.py",
-                                            "twine/first.whl",
-                                            "twine/bar.py",
-                                            "twine/second.whl"])
-    expected = ["twine/first.whl",
-                "twine/second.whl",
-                "twine/foo.py",
-                "twine/bar.py"]
-    assert expected == files
+def test_successful_upload(tmpdir):
+    pypirc = os.path.join(str(tmpdir), ".pypirc")
+    dists = ["tests/fixtures/twine-1.5.0-py2.py3-none-any.whl"]
 
+    with open(pypirc, "w") as fp:
+        fp.write(textwrap.dedent("""
+            [pypi]
+            username:foo
+            password:bar
+        """))
 
-def test_ensure_if_no_wheel_files():
-    files = upload.group_wheel_files_first(["twine/foo.py",
-                                            "twine/bar.py"])
-    expected = ["twine/foo.py",
-                "twine/bar.py"]
-    assert expected == files
+    upload_settings = settings.Settings(
+        repository_name="pypi", sign=None, identity=None, username=None,
+        password=None, comment=None, cert=None, client_cert=None,
+        sign_with=None, config_file=pypirc, skip_existing=False,
+        repository_url=None, verbose=False,
+    )
 
+    stub_response = pretend.stub(
+        is_redirect=False, status_code=201, raise_for_status=lambda: None
+    )
+    stub_repository = pretend.stub(
+        upload=lambda package: stub_response, close=lambda: None
+    )
 
-def test_find_dists_expands_globs():
-    files = sorted(upload.find_dists(['twine/__*.py']))
-    expected = [os.path.join('twine', '__init__.py'),
-                os.path.join('twine', '__main__.py')]
-    assert expected == files
+    upload_settings.create_repository = lambda: stub_repository
 
+    result = upload.upload(upload_settings, dists)
 
-def test_find_dists_errors_on_invalid_globs():
-    with pytest.raises(ValueError):
-        upload.find_dists(['twine/*.rb'])
-
-
-def test_find_dists_handles_real_files():
-    expected = ['twine/__init__.py', 'twine/__main__.py', 'twine/cli.py',
-                'twine/utils.py', 'twine/wheel.py']
-    files = upload.find_dists(expected)
-    assert expected == files
+    # Raising an exception or returning anything truthy would mean that the
+    # upload has failed
+    assert result is None
 
 
 def test_get_config_old_format(tmpdir):
     pypirc = os.path.join(str(tmpdir), ".pypirc")
-    dists = ["tests/fixtures/twine-1.5.0-py2.py3-none-any.whl"]
 
     with open(pypirc, "w") as fp:
         fp.write(textwrap.dedent("""
@@ -79,12 +73,12 @@ def test_get_config_old_format(tmpdir):
         """))
 
     try:
-        upload.upload(dists=dists, repository="pypi", sign=None, identity=None,
-                      username=None, password=None, comment=None,
-                      cert=None, client_cert=None,
-                      sign_with=None, config_file=pypirc, skip_existing=False,
-                      repository_url=None,
-                      )
+        settings.Settings(
+            repository_name="pypi", sign=None, identity=None, username=None,
+            password=None, comment=None, cert=None, client_cert=None,
+            sign_with=None, config_file=pypirc, skip_existing=False,
+            repository_url=None, verbose=False,
+        )
     except KeyError as err:
         assert err.args[0] == (
             "Missing 'pypi' section from the configuration file\n"
@@ -95,6 +89,42 @@ def test_get_config_old_format(tmpdir):
         ).format(pypirc)
 
 
+def test_deprecated_repo(tmpdir):
+    with pytest.raises(exceptions.UploadToDeprecatedPyPIDetected) as err:
+        pypirc = os.path.join(str(tmpdir), ".pypirc")
+        dists = ["tests/fixtures/twine-1.5.0-py2.py3-none-any.whl"]
+
+        with open(pypirc, "w") as fp:
+            fp.write(textwrap.dedent("""
+                [pypi]
+                repository: https://pypi.python.org/pypi/
+                username:foo
+                password:bar
+            """))
+
+        upload_settings = settings.Settings(
+            repository_name="pypi", sign=None, identity=None, username=None,
+            password=None, comment=None, cert=None, client_cert=None,
+            sign_with=None, config_file=pypirc, skip_existing=False,
+            repository_url=None, verbose=False,
+        )
+
+        upload.upload(upload_settings, dists)
+
+    assert err.value.args[0] == (
+        "You're trying to upload to the legacy PyPI site "
+        "'https://pypi.python.org/pypi/'. "
+        "Uploading to those sites is deprecated. \n "
+        "The new sites are pypi.org and test.pypi.org. Try using "
+        "https://upload.pypi.org/legacy/ "
+        "(or https://test.pypi.org/legacy/) "
+        "to upload your packages instead. "
+        "These are the default URLs for Twine now. \n "
+        "More at "
+        "https://packaging.python.org/guides/migrating-to-pypi-org/ ."
+    )
+
+
 def test_skip_existing_skips_files_already_on_PyPI(monkeypatch):
     response = pretend.stub(
         status_code=400,
@@ -108,7 +138,8 @@ def test_skip_existing_skips_files_already_on_PyPI(monkeypatch):
 
 
 def test_skip_existing_skips_files_already_on_pypiserver(monkeypatch):
-    # pypiserver (https://pypi.python.org/pypi/pypiserver) responds with 409
+    # pypiserver (https://pypi.org/project/pypiserver) responds with a
+    # 409 when the file already exists.
     response = pretend.stub(
         status_code=409,
         reason='A file named "twine-1.5.0-py2.py3-none-any.whl" already '
@@ -120,6 +151,21 @@ def test_skip_existing_skips_files_already_on_pypiserver(monkeypatch):
                               package=pkg) is True
 
 
+def test_skip_existing_skips_files_already_on_artifactory(monkeypatch):
+    # Artifactory (https://jfrog.com/artifactory/) responds with 403
+    # when the file already exists.
+    response = pretend.stub(
+        status_code=403,
+        text="Not enough permissions to overwrite artifact "
+             "'pypi-local:twine/1.5.0/twine-1.5.0-py2.py3-none-any.whl'"
+             "(user 'twine-deployer' needs DELETE permission).")
+
+    pkg = package.PackageFile.from_filename(WHEEL_FIXTURE, None)
+    assert upload.skip_upload(response=response,
+                              skip_existing=True,
+                              package=pkg) is True
+
+
 def test_skip_upload_respects_skip_existing(monkeypatch):
     response = pretend.stub(
         status_code=400,
@@ -143,12 +189,7 @@ def test_values_from_env(monkeypatch):
                "TWINE_CERT": "/foo/bar.crt"}
     with helpers.set_env(**testenv):
         cli.dispatch(["upload", "path/to/file"])
-    cli.dispatch(["upload", "path/to/file"])
-    result_kwargs = replaced_upload.calls[0].kwargs
-    assert "pypipassword" == result_kwargs["password"]
-    assert "pypiuser" == result_kwargs["username"]
-    assert "/foo/bar.crt" == result_kwargs["cert"]
-    result_kwargs = replaced_upload.calls[1].kwargs
-    assert None is result_kwargs["password"]
-    assert None is result_kwargs["username"]
-    assert None is result_kwargs["cert"]
+    upload_settings = replaced_upload.calls[0].args[0]
+    assert "pypipassword" == upload_settings.password
+    assert "pypiuser" == upload_settings.username
+    assert "/foo/bar.crt" == upload_settings.cacert
